# -*- coding: utf-8 -*-

"""
    moesif.http_response_catcher
"""

from moesifapi.http.http_call_back import *

class HttpResponseCatcher(HttpCallBack):

    """A class used for catching the HttpResponse object from controllers.

    This class inherits HttpCallBack and implements the on_after_response
    method to catch the HttpResponse object as returned by the HttpClient
    after a request is executed.
    """
    def on_before_request(self, request):
        pass

    def on_after_response(self, context):
        #self.response = context.response
        #print context.response.status_code
        pass





def mask_headers(headers, masks):
    """
    filters any headers (a dictionary) with key that matches those in the
    masks (a list)
    This have a sideeffect, so please make a deepcopy before using.
    """
    if headers is None:
        return headers
    if masks is None:
        return headers

    for member in masks:
        headers.pop(member, None)

    return headers


def mask_body(body, masks):
    """
    recursively removes any element from body (dictionary or lists) that
    have key matches masks. Note, this function have a side effect.
    Please make a deepcopy before using.
    """
    if body is None:
        return body
    if masks is None:
        return body

    if type(body) == list:
        return [mask_body(element, masks) for element in body]

    if type(body) == dict:
        for mask in masks:
            body.pop(mask, None)
        for key in body:
            body[key] = mask_body(body[key], masks)
        return body

    return body



from django.test import SimpleTestCase, TestCase
from django.http import HttpResponse
from django.test import RequestFactory
from django.contrib.auth.models import User
from django.utils import timezone
from moesifdjango.middleware_pre19 import MoesifMiddlewarePre19
from moesifdjango.masks import *
import jsonpickle


class MoesifMiddlewarePre19Test(TestCase):

    def setUp(self):
        self.request_factory = RequestFactory()
        self.user = User.objects.create_user(username='my_user_id', email='test@test.com', password='top_secret')

    def test_get_patched_middlesettings(self):
        def get_response(req):
            response_data = {}
            response_data['testing'] = 'basic'
            response_data['no_data'] = 'simplebody'
            response = HttpResponse(response_data)
            return response

        middleware = MoesifMiddlewarePre19()
        request = self.request_factory.get('/api/acmeinc/items/83738/reviews')
        request.user = self.user
        request.moesif_req_time = timezone.now()
        request._mo_body = None
        response = middleware.process_response(request, get_response(request))
        self.assertEqual(response.status_code, 200)


class UpdateUserTests(SimpleTestCase):

    def testUpdateUser(self):
        MoesifMiddlewarePre19().update_user({
        'user_id': '12345',
        'company_id' : '67890',
        'session_token': 'jkj9324-23489y5324-ksndf8-d9syf8',
        'metadata': {'email': 'abc@email.com', 'name': 'abcde', 'image': '1234'},
        'campaign': {'utm_source': 'Newsletter', 'utm_medium': 'Email'}
        })

class UpdateUsersBatchTest(SimpleTestCase):

    def testUpdateUsersBatch(self):
        MoesifMiddlewarePre19().update_users_batch([{
            'user_id': '12345',
            'company_id' : '67890',
            'metadata': {'email': 'abc@email.com', 'name': 'abcdefg', 'image': '123'}
        }, {
            'user_id': '1234',
            'company_id' : '6789',
            'metadata': {'email': 'abc@email.com', 'name': 'abcdefg', 'image': '123'}
        }])

class UpdateCompanyTest(SimpleTestCase):

    def testUpdateCompany(self):
        MoesifMiddlewarePre19().update_company({
        'company_id': '12345',
        'company_domain': 'acmeinc.com',
        'metadata': {'email': 'abc@email.com', 'name': 'abcde', 'image': '1234'},
        'campaign': {'utm_source': 'Adwords', 'utm_medium': 'Twitter'}
        })

class UpdateCompaniesBatchTest(SimpleTestCase):

    def testUpdateCompaniesBatch(self):
        MoesifMiddlewarePre19().update_companies_batch([{
            'company_id': '12345',
            'company_domain': 'nowhere.com',
            'metadata': {'email': 'abc@email.com', 'name': 'abcdefg', 'image': '123'}
        }, {
            'company_id': '67890',
            'company_domain': 'acmeinc.com',
            'metadata': {'email': 'abc@email.com', 'name': 'abcdefg', 'image': '123'}
        }])


class MaskTests(SimpleTestCase):
    def setUp(self):
        #Nothing to do here
        pass

    def test_mask_headers_with_none_headers(self):
        headers = None
        result = mask_headers(headers, ['mask1'])
        self.assertIsNone(result)

    def test_mask_headers_with_none_and_empty_masks(self):
        headers = {}
        headers['header1'] = 'header1value'
        headers['header2'] = 'header2value'
        masks = None
        result = mask_headers(headers, masks)
        self.assertEqual(headers, result)
        masks = []
        result = mask_headers(headers, [])
        self.assertEqual(headers, result)

    def test_mask_headers_with_masks(self):
        headers = {}
        headers = {}
        headers['header1'] = 'header1value'
        headers['header2'] = 'header2value'
        masks = ['header1', 'header3']
        result = mask_headers(headers, masks)
        self.assertEqual('header2value', result.get('header2'))
        self.assertIsNone(result.get('header1'))

    def test_mask_body_with_none_body(self):
        body = None
        result = mask_body(body, ['mask1'])
        self.assertIsNone(result)

    def test_mask_body_with_none_and_empty_masks(self):
        body = jsonpickle.decode('{"a": [1, 2, 3, {"b": 1}], "b": [1, 2, 3, {"c": 1}]}')
        masks = None
        result = mask_body(body, masks)
        self.assertEqual(body, result)
        masks = []
        result = mask_body(body, masks)
        self.assertEqual(body, result)

    def test_mask_body_with_deep_item(self):
        body = jsonpickle.decode("""
          {
            "a": [1, 2, 3, {"b": 1}],
            "b": [1, 2, 3, {"c": 1}],
            "c": 123,
            "d": "a string"
          }
          """)
        masks = ['c', 'd']
        result = mask_body(body, masks)
        self.assertIsNone(result.get('d'))
        self.assertIsNone(result['b'][3].get('c'))



from moesifapi.models import *
from moesifapi.exceptions.api_exception import *
from moesifapi.api_helper import *


def update_company(company_profile, api_client, DEBUG):
    if not company_profile:
        print('Expecting the input to be either of the type - CompanyModel, dict or json while updating user')
    else:
        if isinstance(company_profile, dict):
            if 'company_id' in company_profile:
                try:
                    api_client.update_company(CompanyModel.from_dictionary(company_profile))
                    if DEBUG:
                        print('Company Profile updated successfully')
                except APIException as inst:
                    if 401 <= inst.response_code <= 403:
                        print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                    if DEBUG:
                        print("Error while updating company, with status code:")
                        print(inst.response_code)
            else:
                print('To update a company, a company_id field is required')

        elif isinstance(company_profile, CompanyModel):
            if company_profile.company_id is not None:
                try:
                    api_client.update_company(company_profile)
                    if DEBUG:
                        print('Company Profile updated successfully')
                except APIException as inst:
                    if 401 <= inst.response_code <= 403:
                        print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                    if DEBUG:
                        print("Error while updating company, with status code:")
                        print(inst.response_code)
            else:
                print('To update a company, a company_id field is required')
        else:
            try:
                company_profile_json = APIHelper.json_deserialize(company_profile)
                if 'company_id' in company_profile_json:
                    try:
                        api_client.update_company(CompanyModel.from_dictionary(company_profile_json))
                        if DEBUG:
                            print('Company Profile updated successfully')
                    except APIException as inst:
                        if 401 <= inst.response_code <= 403:
                            print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                        if DEBUG:
                            print("Error while updating company, with status code:")
                            print(inst.response_code)
                else:
                    print('To update a company, a company_id field is required')
            except:
                print('Error while deserializing the json, please make sure the json is valid')


def update_companies_batch(companies_profiles, api_client, DEBUG):
    if not companies_profiles:
        print('Expecting the input to be either of the type - List of CompanyModel, dict or json while updating users')
    else:
        if all(isinstance(company, dict) for company in companies_profiles):
            if all('company_id' in company for company in companies_profiles):
                try:
                    batch_profiles = [CompanyModel.from_dictionary(d) for d in companies_profiles]
                    api_client.update_companies_batch(batch_profiles)
                    if DEBUG:
                        print('Companies Profile updated successfully')
                except APIException as inst:
                    if 401 <= inst.response_code <= 403:
                        print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                    if DEBUG:
                        print("Error while updating companies, with status code:")
                        print(inst.response_code)
            else:
                print('To update companies, an company_id field is required')

        elif all(isinstance(company, CompanyModel) for company in companies_profiles):
            if all(company.company_id is not None for company in companies_profiles):
                try:
                    api_client.update_companies_batch(companies_profiles)
                    if DEBUG:
                        print('Companies Profile updated successfully')
                except APIException as inst:
                    if 401 <= inst.response_code <= 403:
                        print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                    if DEBUG:
                        print("Error while updating companues, with status code:")
                        print(inst.response_code)
            else:
                print('To update companies, a company_id field is required')
        else:
            try:
                company_profiles_json = [APIHelper.json_deserialize(d) for d in companies_profiles]
                if all(isinstance(company, dict) for company in company_profiles_json) and all(
                                'company_id' in company for company in company_profiles_json):
                    try:
                        batch_profiles = [CompanyModel.from_dictionary(d) for d in company_profiles_json]
                        api_client.update_companies_batch(batch_profiles)
                        if DEBUG:
                            print('Companies Profile updated successfully')
                    except APIException as inst:
                        if 401 <= inst.response_code <= 403:
                            print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                        if DEBUG:
                            print("Error while updating companies, with status code:")
                            print(inst.response_code)
                else:
                    print('To update companies, an company_id field is required')
            except:
                print('Error while deserializing the json, please make sure the json is valid')
